/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.wood;

import java.util.*;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import com.google.common.collect.ImmutableList;
import net.minecraft.block.Block;
import net.minecraft.block.BlockLeaves;
import net.minecraft.block.BlockPlanks;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.entity.Entity;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.api.types.Tree;
import net.dries007.tfc.util.OreDictionaryHelper;

public class BlockLeavesTFC extends BlockLeaves
{
    private static final Map<Tree, BlockLeavesTFC> MAP = new HashMap<>();

    public static BlockLeavesTFC get(Tree wood)
    {
        return MAP.get(wood);
    }

    public final Tree wood;

    public BlockLeavesTFC(Tree wood)
    {
        this.wood = wood;
        if (MAP.put(wood, this) != null) throw new IllegalStateException("There can only be one.");
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(field_176237_a, false)); // TFC leaves don't use CHECK_DECAY, so just don't use it
        field_185686_c = true; // Fast / Fancy graphics works correctly
        OreDictionaryHelper.register(this, "tree", "leaves");
        OreDictionaryHelper.register(this, "tree", "leaves", wood.name());
        Blocks.field_150480_ab.func_180686_a(this, 30, 60);
        func_149675_a(true);
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(field_176237_a, (meta & 0b01) == 0b01);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return (state.func_177229_b(field_176237_a) ? 1 : 0);
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public AxisAlignedBB func_180646_a(IBlockState blockState, @Nonnull IBlockAccess worldIn, @Nonnull BlockPos pos)
    {
        return field_185506_k;
    }

    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World world, BlockPos pos, @Nullable Block blockIn, @Nullable BlockPos fromPos)
    {
        doLeafDecay(world, pos, state);
    }

    @Override
    public void func_180634_a(World worldIn, BlockPos pos, IBlockState state, Entity entityIn)
    {
        //Player will take damage when falling through leaves if fall is over 9 blocks, fall damage is then set to 0.
        entityIn.func_180430_e((entityIn.field_70143_R - 6), 1.0F); // TODO: 17/4/18 Balance fall distance reduction.
        entityIn.field_70143_R = 0;
        //Entity motion is reduced by leaves.
        entityIn.field_70159_w *= 0.1D;
        entityIn.field_70181_x *= 0.1D;
        entityIn.field_70179_y *= 0.1D;
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, field_176237_a);
    }

    @Override
    public void func_180650_b(World worldIn, @Nonnull BlockPos pos, @Nonnull IBlockState state, Random rand)
    {
        doLeafDecay(worldIn, pos, state);
    }

    @Override
    @Nonnull
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        return Item.func_150898_a(BlockSaplingTFC.get(wood));
    }

    @Override
    @Nonnull
    public BlockPlanks.EnumType func_176233_b(int meta)
    {
        // Unused so return whatever
        return BlockPlanks.EnumType.OAK;
    }

    @Override
    public void beginLeavesDecay(IBlockState state, @Nonnull World world, @Nonnull BlockPos pos)
    {
        // Don't do vanilla decay
    }

    @Override
    @Nonnull
    public List<ItemStack> onSheared(@Nonnull ItemStack item, IBlockAccess world, BlockPos pos, int fortune)
    {
        return ImmutableList.of(new ItemStack(this));
    }

    private void doLeafDecay(World world, BlockPos pos, IBlockState state)
    {
        // TFC Leaf Decay
        if (world.field_72995_K || !state.func_177229_b(field_176237_a))
            return;

        List<BlockPos> paths = new ArrayList<>();
        List<BlockPos> pathsToAdd;
        BlockPos.MutableBlockPos pos1 = new BlockPos.MutableBlockPos(pos);
        IBlockState state1;
        paths.add(pos); // Center block

        for (int i = 0; i < wood.maxDecayDistance; i++)
        {
            pathsToAdd = new ArrayList<>();
            for (BlockPos p1 : paths)
            {
                for (EnumFacing face : EnumFacing.values())
                {
                    pos1.func_189533_g(p1).func_189536_c(face);
                    if (paths.contains(pos1.func_185334_h()))
                        continue;
                    state1 = world.func_180495_p(pos1);
                    if (state1.func_177230_c() == BlockLogTFC.get(wood))
                        return;
                    if (state1.func_177230_c() == this)
                        pathsToAdd.add(pos1.func_185334_h());

                }
            }
            paths.addAll(pathsToAdd);
        }

        world.func_175698_g(pos);
    }

    @SideOnly(Side.CLIENT)
    @Override
    public BlockRenderLayer func_180664_k()
    {
        /*
         * This is a way to make sure the leave settings are updated.
         * The result of this call is cached somewhere, so it's not that important, but:
         * The alternative would be to use `Minecraft.getMinecraft().gameSettings.fancyGraphics` directly in the 2 relevant methods.
         * It's better to do that than to refer to Blocks.LEAVES, for performance reasons.
         */
        field_185686_c = Minecraft.func_71410_x().field_71474_y.field_74347_j;
        return super.func_180664_k();
    }

    @SuppressWarnings("deprecation")
    @SideOnly(Side.CLIENT)
    @Override
    public boolean func_176225_a(IBlockState blockState, IBlockAccess blockAccess, BlockPos pos, EnumFacing side)
    {
        /*
         * See comment on getRenderLayer()
         */
        field_185686_c = Minecraft.func_71410_x().field_71474_y.field_74347_j;
        return super.func_176225_a(blockState, blockAccess, pos, side);
    }
}
